
// ImageUtils.java
// Andrew Davison, August 2023, ad@coe.psu.ac.th

/* Utilities, subdivided into the categories:
      * file IO
      * filename related
      * pixels

   Usage:
     > compile ImageUtils.java
*/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;

import java.io.*;
import java.nio.file.*;
import javax.imageio.*;

import java.util.*;


public class ImageUtils
{


  // --------------- file IO ----------------------


  public static BufferedImage loadImage(String fnm)
  // load the image stored in fnm
  {
    BufferedImage image = null;
    try {
      image = ImageIO.read(new File(fnm));
      System.out.println("Loaded " + fnm);
    }
    catch (IOException e) 
    {  System.out.println("Could not load " + fnm); }
    return image;
  }  // end of loadImage()


  public static void saveImage(BufferedImage im, String fnm)
  // save the image into the PNG file called fnm
  {
    try {
      ImageIO.write(im, "png", new File(fnm));
      System.out.println("Saved image to " + fnm); 
    }
    catch(IOException e)
    {  System.out.println("Could not save image to " + fnm);  }
  }  // end of saveImage()



  public static String readFile(String fnm)
  {
    try {
      System.out.println("Reading text from " + fnm);
      return Files.readString(Path.of(fnm));
    }
    catch (IOException e) {
      System.out.println("Could not read " + fnm);
      return null;
    }
  } // end of readFile()



  public static double getDouble(String s)
  { 
    double num = 0;
    try {
      num = Double.parseDouble(s);
    }
    catch (NumberFormatException ex){ 
      System.out.println("Incorrect format for double " + s); 
    }
    return num;
  }  // end of getDouble()



  public static int getInt(String s)
  { 
    int num = 0;
    try {
      num = Integer.parseInt(s);
    }
    catch (NumberFormatException ex){ 
      System.out.println("Incorrect format for int " + s); 
    }
    return num;
  }  // end of getInt()



  // ------------- filename related -------------------


  public static String getName(String fnm)
  // remove path and extension from filename
  {
    Path p = Paths.get(fnm);
    String fileName = p.getFileName().toString();
    return stripExt(fileName);
  }  // end of getName()



  public static String stripExt(String fnm)
  {
    int dotPos = fnm.lastIndexOf('.');
    if (dotPos == -1)
      return fnm;
    else
      return fnm.substring(0, dotPos);
  } // end of stripExt()



  public static String getExt(String fnm)
  {
    int dotPos = fnm.lastIndexOf('.');
    if (dotPos == -1)
      return "txt";
    else
      return fnm.substring(dotPos+1);
  } // end of getExt()



  // -------------------- pixels ----------------------


  public static BufferedImage toGray(BufferedImage im)
  {
    try {
      BufferedImage bwIm = new BufferedImage(im.getWidth(), 
                                             im.getHeight(),
                        BufferedImage.TYPE_BYTE_GRAY);
      Graphics2D g2d = bwIm.createGraphics();
      g2d.drawImage(im, 0, 0, null);
      g2d.dispose();
      return bwIm;
    }
    catch (Exception e)  {
      System.out.println("Unable to convert image to grayscale");
      return im;
    }
  }  // end of toGray() 



  // assume that pixel is an RGB color integer

  public static int getRed(int pixel)
  {  return ((pixel & 0xff0000) >> 16);  }

  public static int getGreen(int pixel)
  {  return ((pixel & 0x00ff00) >> 8);  }

  public static int getBlue(int pixel)
  {  return (pixel & 0x0000ff);  }



  public static int getGray(int pixel)
  // assuming an 8-bit grayscale image
  {  return (pixel & 0xff);  }



  public static int makePixel(int red, int green, int blue)
  {
    return ( ((red & 0xff) << 16) + ((green & 0xff) << 8) +
             (blue & 0xff) );
  }


}  // end of ImageUtils class